#!/bin/bash

# $Id: esrt.sh,v 1.1.2.1.2.1.2.1 2018/10/12 19:23:49 prsurt Exp $

# Use -gt 1 to consume two arguments per pass in the loop (e.g. each
# argument has a corresponding value to go with it).
# Use -gt 0 to consume one or more arguments per pass in the loop (e.g.
# some arguments don't have a corresponding value to go with it such
# as in the --default example).
# note: if this is set to -gt 0 the /etc/hosts part is not recognized ( may be a bug )
while [[ $# -gt 1 ]]
do
key="$1"

case $key in
    -h|--host)
    HOST="$2"
    shift # past argument
    ;;
    -p|--port)
    PORT="$2"
    shift # past argument
    ;;
    -q|--query)
    QUERY="$2"
    shift # past argument
    ;;
    -f|--field)
    FIELD="$2"
    shift # past argument
    ;;
    -i|--index)
    IDX="$2"
    shift # past argument
    ;;
    -s|--stats)
    STATS="$2"
    shift # past argument
    ;;
    -b|--bins)
    BUCKETS="$2"
    shift # past argument
    ;;
    --default)
    DEFAULT=YES
    ;;
    *)
            # unknown option
    ;;
esac
shift # past argument or value
done
if [[ ${PORT} == "" ]]; then
PORT=9200
fi
if [[ ${QUERY} == "" ]]; then
QUERY=*:*
fi
if [[ ${FIELD} == "" ]]; then
AGG='{}'
else
AGG='{"size":0,"aggs":{"'"${FIELD}"'":{"terms":{"field":"'"${FIELD}"'","size":100}}}}' 
fi
if [[ ${STATS} == "" ]]; then
echo ""
else
AGG='{"query":{"bool":{"must":[],"must_not":[],"should":[{"match_all":{}}]}},"from":0,"size":0,"sort":[],"aggs":{"f-1":{"stats":{"field":"'"${STATS}"'"}}},"version":true}'
fi
if [[ ${BUCKETS} == "" ]]; then
echo ""
else
AGG='{"query":{"bool":{"must":[],"must_not":[],"should":[{"match_all":{}}]}},"from":0,"size":0,"sort":[],"aggs":{"f-2":{"date_histogram":{"field":"CMX_CREATE_TIMESTAMP","interval":"'"${BUCKETS}"'"}}},"version":true}'
fi
echo HOST  = "${HOST}"
echo PORT     = "${PORT}"
echo QUERY    = "${QUERY}"
echo FIELD    = "${FIELD}"
echo IDX      = "${IDX}"
echo " "
echo "Response:" \
$(time curl -X POST -d ${AGG}  "http://${HOST}:${PORT}/${IDX}/_search?q=${QUERY}"
 )
