/* $Id: jose.sas,v 1.1.4.2.2.1.16.1 2018/08/28 19:31:41 rosacc Exp $ */

/*
 * Logging
 *
 * %let RPT_VERBOSE = 13;
 *
 * Logging can be voluminous.  The level can be set from 1 to 13.
 *
 * It can also be set on a per-transaction basis by setting smh_source to "TRACE".
 *
 * It could also be set at runtime by sending a SCMD message the SAS OSE (below).
 * As of version 41M1, this would have to be done programmatically (from behind
 * JMX); in-band commands other than ping (cmd_code=0) are no longer accepted.
 *
 *  scc_cmd_code = 1
 *  scc_cmd_info[1,8] = "VERBOSE=";
 *  scc_cmd_info[9,2] = level in Z2. Format
 */

%macro sysget_not_empty(key);

    %local x;
    %let x=%sysget(&key);

    %put &key.=&x;

    %if %sysevalf(%superq(x)=, boolean) %then %do;

        %put &key is not set;
        abort;

    %end;

    &x

%mend sysget_not_empty;

options %sysget(OPTIONS);

%let MSG_INPUT_ENCODING  = %sysget_not_empty(OSE_MSG_ENCODING);
%let MSG_OUTPUT_ENCODING = %sysget_not_empty(OSE_MSG_ENCODING);

%let TRX_COMTYPE = %sysget_not_empty(METHOD);

%let OSE_SCOREMATCH = %sysget(OSE_SCOREMATCH);

%macro init_jose;

    %if &TRX_COMTYPE = TCP %then %do;

        %global SERVERPORTNUM;
        %global SERVERHOSTNAME;
        %let SERVERPORTNUM=%sysget_not_empty(PORT);
        %let SERVERHOSTNAME=localhost;

    %end;
    %else %if &TRX_COMTYPE = SHM %then %do;

        %global SERVERSHMKEY;
        %global LOGFILE_PATTERN;
        %let SERVERSHMKEY=%sysget_not_empty(SHM_KEYS);
        %let LOGFILE_PATTERN = %sysget_not_empty(LOGS_HOME)/%sysget_not_empty(LOG_PATTERN);

    %end;
    %else %do;

        %put "bogus TRX_COMTYPE: &TRX_COMTYPE";
        abort;

    %end;

%mend init_jose;

%init_jose

libname CRULES  "%sysget_not_empty(CRULES)";
libname CMODELS "%sysget_not_empty(CMODELS)";

filename RULES  "%sysget_not_empty(RULES)";
filename LRULES "%sysget_not_empty(LRULES)";

%include "%sysget_not_empty(SCRIPT)";

/* XXX is this necessary, doesn't it hide problems? */
*);*/;/*'*/ /*\"*/;   /* Close any syntax errors */
