jQuery.sap.declare('sas.fscf.CustomBootTask');

jQuery.sap.require('sas.hc.ui.core.Core');
jQuery.sap.require('sas.hc.ui.core.BootTask');
jQuery.sap.require('sap.ui.core.format.NumberFormat');
jQuery.sap.require('sap.ui.core.format.DateFormat');
jQuery.sap.require('sap.ui.unified.Calendar');

sas.hc.ui.core.BootTask.extend("sas.fscf.CustomBootTask", {
    name: 'SAS-fscf-CustomBootTask'
});

sas.fscf.CustomBootTask.prototype.run = function () {
	// Do stuff here to participate in the boot strap
	// Generally stuff you want to do before application view is instantiated.

    jQuery.sap.includeStyleSheet("resources/sas/fscf/fscf.css");

    sas.config.initialSessionMaxInActiveInterval = sas.config.sessionMaxInActiveInterval;
    sas.config.initialSessionTimeoutWarningDuration = sas.config.sessionTimeoutWarningDuration;
    sas.fscf.NLS_BASE = 'sas.fscf';
    sas.fscf.MESSAGE_BUNDLE_PATH = 'resources/sas/fscf/messagebundle.properties';
    sas.fscf.rb = sap.ui.getCore().getLibraryResourceBundle(sas.fscf.NLS_BASE);
	sas.ajax({url:'rest/users/@currentUser'})
	  .done(function(oData){
	      sas.fscf.currentUser = oData;
	}).fail(function(oObj,textStatus){			
       console.info("Failed to get current user.");
	});
    sas.fscf.floatFormatter = sap.ui.core.format.NumberFormat.getFloatInstance({maxFractionDigits: 2});
    sas.fscf.floatFormatterTranData = sap.ui.core.format.NumberFormat.getFloatInstance();
    sas.fscf.floatFormatterTranData.oFormatOptions.pattern = sas.config.get('fscf').decimalFormat;	// S1468348: set pattern after construction to avoid bug where groupingType can go to indian incorrectly (instead of arabic)
    sas.fscf.percentFormatter = sap.ui.core.format.NumberFormat.getPercentInstance({minFractionDigits:2,maxFractionDigits:2});
    sas.fscf.currencyFormatter = sap.ui.core.format.NumberFormat.getCurrencyInstance();
    sas.fscf.currencyFormatter.oFormatOptions.pattern = sas.config.get('fscf').currencyFormat;	// S1468348: set pattern after construction to avoid bug where groupingType can go to indian incorrectly (instead of arabic)
    sas.fscf.dateTimeFormatter = sap.ui.core.format.DateFormat.getDateTimeInstance({pattern: sas.config.get('fscf').dateTimeFormat});
    sas.fscf.dateFormatter = sap.ui.core.format.DateFormat.getDateInstance({pattern: sas.config.get('fscf').dateFormat});
    sas.fscf.timeFormatter = sap.ui.core.format.DateFormat.getTimeInstance({pattern: sas.config.get('fscf').timeFormat});
    sas.fscf.ruleEnableGuided = sas.config.get('fscf').ruleEnableGuided;
    sas.fscf.enableDynamicLookupLists = sas.config.get('fscf').enableDynamicLookupLists;
    sas.fscf.RULE_APPROACH_GUIDED = 0;
    sas.fscf.RULE_APPROACH_CODING = 1;
    sas.fscf.MESSAGE_TYPE_INFO = 0;
    sas.fscf.MESSAGE_TYPE_COMPLETE = 1;
    sas.fscf.MESSAGE_TYPE_WARNING = 2;
    sas.fscf.MESSAGE_TYPE_ERROR = 3;
	sas.fscf.inlineMessageClasses = ["inlineMessageInfo","inlineMessageComplete","inlineMessageWarning","inlineMessageError"];
	sas.fscf.inlineMessageIconContents = ["&#xf330;","&#xf160;","&#xf018;","&#xf22f;"];
	sas.fscf.showInlineMessage =  function(parentId, type, message, bCenter, customIconContent, bTransparent) {
		var $p = $("#" + parentId);
    	if ($p.length == 0) {
    		return;
    	}
		$p.find('.inlineMessage').remove();
    	if (type == undefined) {
    		type = sas.fscf.MESSAGE_TYPE_INFO;
    	}
		var markup = [];
		markup[markup.length] = '<div class="inlineMessage ' + sas.fscf.inlineMessageClasses[type] + ((bTransparent) ? ' inlineMessageTransparent' : '') + '">';
		markup[markup.length] = '<div class="inlineMessageIcon">';
		markup[markup.length] = '<span class="sapUiBtnIco sapUiBtnIcoL sapUiIcon" style="font-family: sas-icons;" data-sap-ui-icon-content="' + ((customIconContent) ? customIconContent : sas.fscf.inlineMessageIconContents[type]) + '"></span>';
		markup[markup.length] = '</div>';
		markup[markup.length] = '<div class="inlineMessageMessage">';
		markup[markup.length] = message;
		markup[markup.length] = '</div>';
		if (!bTransparent) {
			markup[markup.length] = '<div class="inlineMessageClose">';
			markup[markup.length] = '<a href="#" title="' + sas.fscf.rb.getText("fcm.application.close.txt") + '" onclick="this.parentNode.parentNode.style.display=\'none\';">';
			markup[markup.length] = '<span class="sapUiBtnIco sapUiBtnIcoL sapUiIcon" style="font-family: sas-icons;" data-sap-ui-icon-content="&#xf10c;"></span>';
			markup[markup.length] = '</a>';
			markup[markup.length] = '</div>';
		}
		markup[markup.length] = '</div>';
        $p.prepend(markup.join(''));
        if (bCenter) {
        	var $m = $p.find('.inlineMessage').first();
        	$m.css({position : 'absolute', zIndex : '5', margin: '0px', left: '33%', top: '25%', maxWidth: '33%'});
        }
    };
    sas.fscf.hideInlineMessage = function(parentId) {
    	var $p = $('#' + parentId);
    	if ($p.length == 0) {
    		return;
    	}
		$p.find('.inlineMessage').remove();
    };

/*    sas.fscf.helpUrlLookup = {
    	"ITabReports":"a_reports.htm",
    	"ITabStrategies":"a_strategies.htm",
    	"ITabTemplates":"a_templates.htm",
    	"ITabNotifications":"a_notifications.htm",
    	"ITabGroups":"a_groupTab.htm",
    	"ITabUsers":"a_users.htm",
    	"ITabModel":"a_modelTab.htm",
    	"ITabAnalystWorkstation":"fcmAppMain.htm",
    	"ITabPreference":"a_preferences.htm",
    	"ITabRules":"r_rulesView.htm",
    	"ITabConsole":"r_consoleView.htm",
    	"ITabLoader":"a_incrloader.htm"
    };*/
    sas.fscf.resizeContainers = function () {
		$("#sas_hc_content .sasFCHContent").css("overflow", "hidden");
		$("#sas_hc_content .sasFCHContent").height(
				$("#sas_hc_content .sasFCHContent").parent().height() -
				$("#sas_hc_content .sasFHCHeader").height());
    	$("#sas_hc_content .sapMITBContainerContent").height(
    			$("#sas_hc_content .sasFCHContent").height() -
    			$("#sas_hc_content .sapMITH").height() - 3);
    	var $iframe = $("iframe[name='overview']");
    	if ($iframe.length) {
    		var h = Math.floor($iframe.parent().height() -
    				($iframe.position().top - $iframe.parent().position().top)) - 6;
    		$iframe.css("height", h + "px")
    	}
    	$iframe = $("iframe[name='detail']");
    	if ($iframe.length) {
    		var h = Math.floor($iframe.parent().height() -
    				($iframe.position().top - $iframe.parent().position().top)) - 6;
    		$iframe.css("height", h + "px")
    	}
    };
    $(document).ready(sas.fscf.resizeContainers);
    $(window).resize(sas.fscf.resizeContainers);
	sas.fscf.sessionBindIFrames = function () {
		$("iframe").load(function (e) {
			sas.fscf.sessionUserActivity();
			$(e.target).contents().click(function (e) {
				sas.fscf.sessionUserActivity();
				return true;
			});
		});
	};
	sas.fscf.setCookie = function(name,value,expireDate)
	{
		if(!value || value == '')
		{
			sas.fscf.removeCookie(name,'');
			return;
		}

		var theDate;
		if(expireDate)
		{
			theDate = expireDate;
		}
		else
		{
			theDate = new Date();
			theDate.setDate(theDate.getDate()+1);
		}

		document.cookie = name + "=" + escape(value) + ";expires=" + theDate.toGMTString();
	};
	sas.fscf.getCookie = function(name,defaultValue)
	{
		var cookies = document.cookie.split("; ");
		for(var i=0; i < cookies.length; i++)
		{
			var theCookie = cookies[i].split("=");
			if (name == theCookie[0])
				return unescape(theCookie[1]);
		}

		return defaultValue;
	};
	sas.fscf.removeCookie = function(name,value)
	{
		var theDate = new Date();
		theDate.setDate(theDate.getDate()-1);
		document.cookie = name + "=" + escape(value) + ";expires=" + theDate.toGMTString();
	};
	sas.fscf.getNYearsFromNow = function(n)
	{
		var d = new Date();
		d.setFullYear(d.getFullYear() + n);
		return d;
	};
	sas.fscf.userHasPrivilege = function(privilege)
	{
		// Someone could ask for has privilege before the asynchronous ajax call to the rest feed
		// that sets sas.fscf.currentUser (above) is finished, so this does up to 15 time-delayed checks
		// for sas.fscf.currentUser if needed. Once sas.fscf.currentUser is done being set once,
		// the time-delayed checks won't ever be needed/used. Typically only 1 time-delayed check is
		// ever needed if any.
		var doCheck = function(privilege) {
			if (sas.fscf.currentUser) {
				for (var i = 0; i < sas.fscf.currentUser.roles.length; i++) {
					for (var j = 0; j < sas.fscf.currentUser.roles[i].privileges.length; j++) {
						if (sas.fscf.currentUser.roles[i].privileges[j].name == privilege) {
							return true;
						}
					}
				}
			}
			return false;
		};
		
		if (sas.fscf.currentUser) {
			return doCheck(privilege);
		} else {
			var count = 0;
			var timer = setInterval(function () {
				if (sas.fscf.currentUser || (count > 15)) {
	    			clearInterval(timer);
	    			return doCheck(privilege);
				} else {
					count++;
				}
			}, 500);			
		}
	};
};

// register with sas hc
(function () {
    sas.hc.ui.getCore().registerTask(new sas.fscf.CustomBootTask());
}())
