sap.ui.jsview("sas.fscf.views.RelatedRows", {

	getControllerName: function() {
		return "sas.fscf.views.RelatedRows";
	},
	
    _bExplore: false,
    
    isExplore: function() {
    	return this._bExplore;
    },
    
    setExplore: function(bExplore) {
    	this._bExplore = bExplore;
    },

	_fields: [],
	_fieldVariableTypes: [],
	_alertTypes: [],
	_analystListsFieldTypes: [],
	
	createContent: function(oController) {		
	},
	
	lazyLoadContent: function(oController) {
      if (this.getContent().length == 0) {
		jQuery.sap.require("sas.hc.ui.table.Table");
		jQuery.sap.require("sas.hc.ui.table.Column");
		jQuery.sap.require("sap.ui.model.json.JSONModel");
    	jQuery.sap.require("sas.hc.ui.unified.Menu");
        jQuery.sap.require("sas.hc.ui.unified.MenuItem");

		var _this = this;
		
		this._bCanCreateAlert = sas.fscf.userHasPrivilege("MANALERT");
		this._bCanAddToAnalystList = sas.fscf.userHasPrivilege("ADDDATLIST");
		this._bCanRemoveFromAnalystList = sas.fscf.userHasPrivilege("RMDATLIST");
		
    	this.addStyleClass("cfRelatedRowsView");    	

		var fieldtypesUrl = "rest/";
		if (this.isExplore()) {
			fieldtypesUrl += "businessunits/fieldtypes";
		} else {
			fieldtypesUrl += "businessunits/" + sas.fscf.getCurrentPageView().jsonRule.businessUnitId + "/fieldtypes?counts=true";
			if (sas.fscf.getCurrentPageView().jsonRule.transactionTypeId &&
	       		sas.fscf.getCurrentPageView().jsonRule.transactionTypeId != "-1") {
	   			fieldtypesUrl += "&txnType=" + sas.fscf.getCurrentPageView().jsonRule.transactionTypeId;
			}
		}
        this._fieldtypesModel = new sap.ui.model.json.JSONModel(fieldtypesUrl);

		var model = new sap.ui.model.json.JSONModel({});

		var table = new sas.hc.ui.table.Table({
			id: this.sId + '_TransactionTable', 
			editable: true,
			visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto
		});

		table.setModel(model);
		table.bindRows("/items");
		table.attachBeforeColumnsDialogClose(function(oEvent) {
			if (oEvent.mParameters.acceptedChange) {
				_this._fields = [];
				var colLimit = 200;
				var colLimitExceeded = false;
				oEvent.oSource.getColumns().forEach(function(col) {
					if (col.getVisible()) {
						if (_this._fields.length < colLimit) {
							_this._fields[_this._fields.length] = col.getSortProperty();
						} else {
							colLimitExceeded = true;
							col.setVisible(false);
						}
					}
				});
				if (colLimitExceeded) {
    		  		sas.fscf.showInlineMessage(sas.fscf.getCurrentPageView().sId + "GuidedViewBorderLayoutCenter1", sas.fscf.MESSAGE_TYPE_WARNING, sas.fscf.rb.getText("rules.rulesPage.guidedView.relatedRows.maxColumnsExceeded.fmt.txt", colLimit));
				}
				_this.update();
				if (!sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties) {
					sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties = {};
				}
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns = _this._fields.join(" ");
				sas.fscf.getCurrentPageView().setDirty(true);				
			}
		});
		table.attachColumnMove(function(oEvent) {
			var i = $.inArray(oEvent.mParameters.column.getSortProperty(), _this._fields);
			if (i !== -1) {
				_this._fields.splice(i, 1); // remove the field
				_this._fields.splice(oEvent.mParameters.newPos, 0, oEvent.mParameters.column.getSortProperty()); // add it back in the new position
				if (!sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties) {
					sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties = {};
				}
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns = _this._fields.join(" ");
				sas.fscf.getCurrentPageView().setDirty(true);				
			}
		});
		
		if (this.isExplore()) {
	    	sas.ajax({url:'rest/alerttypes'})
			  .done(function(oData){ 
				  _this._alertTypes = oData.items;
			}).fail(function(oObj,textStatus){			
		         console.info("Failed to get alert types.");
			});			
			
	    	sas.ajax({url:'rest/analystlists/fieldtypes'})
			  .done(function(oData){ 
				  _this._analystListsFieldTypes = oData.items;
			}).fail(function(oObj,textStatus){			
		         console.info("Failed to get alert types.");
			});			
			
			var oMenu = new sas.hc.ui.unified.Menu(table.sId + "_Menu");
			// set name attribute
	    	oMenu.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.Menu.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.Menu.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "Menu");
	    	};
	        table.setAggregation("contextMenu", oMenu);
	        table.attachCellContextmenu(function(oEvent) {
				_this._contextField = oEvent.oSource.getColumns()[parseInt(oEvent.mParameters.columnIndex)].getSortProperty();
				_this._contextAlertType = _this.getAlertTypeForField(_this._contextField);
				
				// table could be scrolled down, so factor in first visible row
				var row = oEvent.oSource.getRows()[oEvent.mParameters.rowIndex - oEvent.oSource.getFirstVisibleRow()];
				if (row) {
					_this._contextValue = row.getCells()[parseInt(oEvent.mParameters.columnIndex)].getText();
				} else {
					_this._contextValue = "";
				}
				if (row) {
					// table could be sorted, so factor in index from binding
					var rowData = row.getModel().getData().items[table.getBinding("rows").aIndices[row.getIndex()]];
					if (rowData) {
						_this._contextTxnId = "" + rowData["CMX_TRAN_ID"];
						_this._contextTxnAcronym = rowData["SMH_ACCT_TYPE"] + rowData["SMH_ACTIVITY_TYPE"]; 
					} else {
						_this._contextTxnId = "";
						_this._contextTxnAcronym = ""; 
					}
				} else {
					_this._contextTxnId = "";
					_this._contextTxnAcronym = ""; 
				}
				var bEnabled = table.getModel().getData() && (table.getModel().getData().count > 0) && (_this._contextTxnId.length > 0);
       		    oMenuItem.setEnabled(_this._bCanCreateAlert && bEnabled);
       		    oMenuItemAddToList.setEnabled(_this._bCanAddToAnalystList && bEnabled);
       		    oMenuItemRemoveFromList.setEnabled(_this._bCanRemoveFromAnalystList && bEnabled);
       		});

	        var oMenuItem = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_CreateAlertMenuItem",
	            text: sas.fscf.rb.getText("rules.rulesPage.guidedView.form.actions.CreateAnAlert.txt"),
	            enabled: false
	        });
	        oMenuItem.attachSelect(function(oEvent) {
	        	_this.getActionConfiguration().open();
	        });
			// set name attribute
	    	oMenuItem.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "CreateAlertMenuItem");
	    	};
	        oMenu.addItem(oMenuItem);

	        var oMenuItemAddToList = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_AddToListMenuItem",
	            text: sas.fscf.rb.getText("rules.rulesPage.guidedView.relatedRows.actions.AddToList.txt"),
	            enabled: false,
                startsSection: true
	        });
	        oMenuItemAddToList.attachSelect(function(oEvent) {
	        	_this.getAnalystListDialog(true).open();
	        });
			// set name attribute
	        oMenuItemAddToList.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "AddToListMenuItem");
	    	};
	        oMenu.addItem(oMenuItemAddToList);

	        var oMenuItemRemoveFromList = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_RemoveFromListMenuItem",
	            text: sas.fscf.rb.getText("rules.rulesPage.guidedView.relatedRows.actions.RemoveFromList.txt"),
	            enabled: false
	        });
	        oMenuItemRemoveFromList.attachSelect(function(oEvent) {
	        	_this.getAnalystListDialog(false).open();
	        });
			// set name attribute
	        oMenuItemRemoveFromList.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "RemoveFromListMenuItem");
	    	};
	        oMenu.addItem(oMenuItemRemoveFromList);
		}
		
        this.addContent(table);
        
        if (this.isExplore()) {
        	setTimeout(function() {
            	_this.update();
        	}, 1000);
        }
      }
	},
	
	update: function() {
		// do nothing if the Related Rows tab is not the current tab
		if (sas.fscf.getCurrentPageView().getGuidedView().getCurrentTab &&
			sas.fscf.getCurrentPageView().getGuidedView().getCurrentTab().indexOf("RelatedRowsTab") == -1) {
			//console.info("Related Rows not current tab; did not update");
			return;
		}
		
		//console.info("updating Related Rows");
		var _this = this;
		this.setBusy(true);

		var table = sap.ui.getCore().byId(_this.sId + '_TransactionTable');
	  	  
		// add the columns if they have not already been added
		if (table.getColumns().length == 0) {
			var defaultRelatedRowsColumns;
			if (this.isExplore()) {
				defaultRelatedRowsColumns = sas.config.get('fscf').exploreDefaultRelatedRowsColumns;
			} else {
				defaultRelatedRowsColumns = sas.config.get('fscf').defaultRelatedRowsColumns;
			}
			if (sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties && 
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns && 
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns.length > 0) {
				defaultRelatedRowsColumns = sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns;
			} 
			if (defaultRelatedRowsColumns) {
				// add the initially visible columns
				this._fields = defaultRelatedRowsColumns.split(" ");
				this._fieldVariableTypes = [];
				for (var i = 0; i < this._fields.length; i++) {
					this._fieldVariableTypes[i] = this.getFieldVariableType(this._fields[i]);
		    		if (this._fieldVariableTypes[i] != null) {
						table.addColumn(
								new sas.hc.ui.table.Column({
									label: _this.getHeaderLabel(this._fields[i]),
									template: new sas.hc.m.Text({
										text: {path: this._fields[i],
		                                       formatter: this.getTextFormatterForVariableType(this._fieldVariableTypes[i])
		                                      },
										wrapping: false
									}),
									sortProperty: this._fields[i],
									visible: true,
									width: "200px"
								})
						);
		    		}
				}
			}
			
			if (this._fieldtypesModel.getData().items) {
				// add the initially not-visible columns
				var item;
				for (var i = 0; i < this._fieldtypesModel.getData().items.length; i++) {
					item = this._fieldtypesModel.getData().items[i];
					if ($.inArray(item.altName, this._fields) === -1) {
						table.addColumn(
								new sas.hc.ui.table.Column({
									label: _this.getHeaderLabel(item.altName),
									template: new sas.hc.m.Text({
										text: {path: item.altName,
		                                       formatter: this.getTextFormatterForVariableType(item.variableType)
											  },
										wrapping: false
									}),
									sortProperty: item.altName,
									visible: false,
									width: "200px"
								})
						);
					}
				}
			}
		}
	  
		// update the rows and re-bind the data model
		var schema = null;
		if (_this.isExplore()) {
			schema = "esdb";
		}
        sas.ajax({url:sas.fscf.getQueryUrlFromRuleModel("search", schema, _this.padFields(_this._fields)), dataType:"json"})
          .done(function(oData){
        	  table.getModel().setData(oData);
  			  _this.setBusy(false);
        }).fail(function(oObj,textStatus){
        	console.info("an error occurred getting the related rows");
			_this.setBusy(false);
        });
	},
	
	getHeaderLabel: function(fieldName) {
		var oLabel = new sas.hc.m.Label({
			text: fieldName
		});
		if (this.getAlertTypeForField(fieldName) != null) {
			oLabel._alertable = true;
		}
		oLabel._inAnalystList = this.isInAnalystList(fieldName);
		oLabel.onAfterRendering = function() {
           	if (sas.hc.m.Label.prototype.onAfterRendering) {
           		sas.hc.m.Label.prototype.onAfterRendering.apply(this);
           	}

           	if (this._inAnalystList) {
    			$label = $("#" + this.sId);
		    	$label.find('.cfHeaderInAnalystList').remove(); // remove any icon that might already be there
    			$("<span class=\"sapUiBtnIco sapUiBtnIcoL sapUiIcon cfHeaderInAnalystList\" style=\"font-family: sas-icons;\" data-sap-ui-icon-content=\"&#xf0bb;\" title=\"" + sas.fscf.rb.getText("rules.rulesPage.guidedView.relatedRows.inList.txt") + "\"></span>").insertAfter($label.find(".sasMLabelText"));
           	}
           	if (this._alertable) {
    			$label = $("#" + this.sId);
		    	$label.find('.cfHeaderAlertable').remove(); // remove any icon that might already be there
    			$("<span class=\"sapUiBtnIco sapUiBtnIcoL sapUiIcon cfHeaderAlertable\" style=\"font-family: sas-icons;\" data-sap-ui-icon-content=\"&#xf018;\" title=\"" + sas.fscf.rb.getText("rules.rulesPage.guidedView.relatedRows.alertable.txt") + "\"></span>").insertAfter($label.find(".sasMLabelText"));
           	}
    	};
    	return oLabel;
	},
    
    padFields: function(fields) {
    	if (this.isExplore()) {
    		// make sure CMX_TRAN_ID, SMH_ACCT_TYPE & SMH_ACTIVITY_TYPE are always in fields, 
    		// as their values are needed to determine _contextTxnId & _contextTxnAcronym for Create an Alert
    		var newArray = fields.slice(0); // clone the array
    		if ($.inArray("CMX_TRAN_ID", newArray) === -1) {
    			newArray[newArray.length] = "CMX_TRAN_ID";
    		} 		
    		if ($.inArray("SMH_ACCT_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACCT_TYPE";
    		} 		
    		if ($.inArray("SMH_ACTIVITY_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACTIVITY_TYPE";
    		} 		
        	return newArray;
    	}
    	return fields;
    },
	
	getAlertTypeForField: function(fieldName) {
		for (var i = 0; i < this._alertTypes.length; i++) {
			if (this._alertTypes[i].AlertableEntityField.toLowerCase() == fieldName.toLowerCase()) {
				return this._alertTypes[i];
			}
		}
		return null;
	},
	
	isInAnalystList: function(fieldName) {
		for (var i = 0; i < this._analystListsFieldTypes.length; i++) {
			if (this._analystListsFieldTypes[i].name.toLowerCase() == fieldName.toLowerCase()) {
				return true;
			}
		}
		return false;
	},
	
    getFieldVariableType: function(fieldName) {
    	if (this._fieldtypesModel.getData().items) {
        	for (var i = 0; i < this._fieldtypesModel.getData().items.length; i++) {
        		if (this._fieldtypesModel.getData().items[i].altName == fieldName) {
        			return this._fieldtypesModel.getData().items[i].variableType;
        		}
        	}
    	}
    	return null;
    },
    
    getTextFormatterForVariableType: function(variableType) {
    	if (variableType == "Numeric") {
    		return function(val) {
    			if (val == undefined || val == null) {
    				return val;
    			}
    			var num = Number(val);
    			if (isNaN(num) || num % 1 === 0) {		// don't format integers
    				return val;
    			}
        		return sas.fscf.floatFormatterTranData.format(val);
    		};
    	} else if (variableType == "Currency") {
       		return function(val) {
       			if (val == undefined || val == null) {
       				return val;
       			}
       			var num = Number(val);
       			if (isNaN(num)) {
       				return val;
       			}
           		return sas.fscf.currencyFormatter.format(val);
       		};
    	} else if (variableType == "DateTime") {
    		return function(val) {    			
    			if (val == undefined || val == null) {
    				return val;
    			}
    			var date = new Date(val);
    			if (isNaN(date.getTime())) {
    				return val;
    			}
        		return sas.fscf.dateTimeFormatter.format(date, true);	// true means use UTC
    		};    		
    	} else if (variableType == "Date") {
    		return function(val) {
    			if (val == undefined || val == null) {
    				return val;
    			}
    			var date = new Date(val);
    			if (isNaN(date.getTime())) {
    				return val;
    			}
        		return sas.fscf.dateFormatter.format(date, true);	// true means use UTC
    		};    		
    	} else if (variableType == "Time") {
    		return function(val) {
    			if (val == undefined || val == null) {
    				return val;
    			}
    			var date = new Date("2000/01/01 " + val);
    			if (isNaN(date.getTime())) {
    				date = new Date(val);
        			if (isNaN(date.getTime())) {
    				    return val;
    			    }
    			}
        		return sas.fscf.timeFormatter.format(date, true);	// true means use UTC
    		};    		
    	} else {
    		return function(val) {
        		return val;
    		};    		
    	}
    },
    
    getPalette: function() {
    	return sap.ui.getCore().byId(this.sId.substr(0, this.sId.indexOf("RelatedRowsView")) + "PaletteView");
    },
    
    getActionConfiguration: function(){
    	var view = sap.ui.jsfragment({
            type: sap.ui.core.mvc.ViewType.JS,
            id: this.sId + "AlertCreationConfigurationDialog",
            fragmentName: "sas.fscf.views.fragments.AlertCreationConfigurationDialog"
        });
    	view.setExplore(true);
    	view.setExploreContext({"txnId":this._contextTxnId, "txnAcronym":this._contextTxnAcronym, "fieldName":this._contextField, "fieldValue":this._contextValue, "alertType":this._contextAlertType});
    	return view.lazyLoadContent();
    },    
    
    getAnalystListDialog: function(bAdd){
    	var view = sap.ui.jsfragment({
            type: sap.ui.core.mvc.ViewType.JS,
            id: this.sId + "AnalystListDialog",
            fragmentName: "sas.fscf.views.fragments.AnalystListDialog"
        });
    	view.setAdd(bAdd);
    	view.setExplore(true);
    	view.setExploreContext({"txnId":this._contextTxnId, "txnAcronym":this._contextTxnAcronym, "fieldName":this._contextField, "fieldValue":this._contextValue, "alertType":this._contextAlertType});
    	return view.lazyLoadContent();
    }    
});