function showRuleEstimates(action,nodeId) {
	if (!nodeId) {
		var selectedNodes = top.frames['overview']._rulesFolderTree.getSelectedNodes();
		var node = selectedNodes[0];
		nodeId = node.getId();
	}
	setCookieForTab(nodeId);
	if (nodeId == "estimates") {
		return;  //  do nothing on the folder
	}
	var target = action + "?method=listRuleEstimates&showAll=" + isAllEstimationsSelected();
	parent.frames[ "detail" ].location.href = target;
}

// Initialize rule list for estimation creation
function initializeEstimationCreator()
{
  showHideRowsPerRuleState();
  if (window.initializeBumper) {
	  initializeBumper();
   }
  initializeTuningValues();
}

// Set tuning values to from a cookie if present
function initializeTuningValues()
{
    setTuningValueFromCookie("samplePercent");
    setTuningValueFromCookie("splitTransactions");
    setTuningValueFromCookie("sampleKey");
    setTuningValueFromCookie("useMultiThreading");
    setTuningValueFromCookie("sampleThreads");
    setTuningValueFromCookie("cacheUserVars");
    setTuningValueFromCookie("sortInSas");
}

// Set tuning value from cookie if present
var ESTIMATION_TUNING_VALUE_COOKIE_PREFIX = "estimationTuningValue_";
function setTuningValueFromCookie(tagId)
{
    var cookieName = ESTIMATION_TUNING_VALUE_COOKIE_PREFIX + tagId;
    var cookieValue = getCookie(cookieName);
    if (cookieValue && cookieValue!='') {
        var tag = document.getElementById(tagId);
        if (tag) {
            tagName = tag.tagName.toUpperCase();
            if (tagName=='SELECT') {
                if (tag.selectedIndex!=cookieValue)
                    showAdvanced();
                tag.selectedIndex = cookieValue;
            }
            else if (tagName=='INPUT') {
                var cookieValueBool = (cookieValue.toUpperCase()=='TRUE');
                if (tag.checked!=cookieValueBool)
                    showAdvanced();
                tag.checked = cookieValueBool;
            }
        }
    }
}

function setCookieFromTuningValue(tagId)
{
    var cookieName = ESTIMATION_TUNING_VALUE_COOKIE_PREFIX + tagId;
    var tag = document.getElementById(tagId);
    if (tag) {
        var yearHence = new Date(); // S1062319
        yearHence.setDate(yearHence.getDate() + 365); // S1062319
        var tagName = tag.tagName.toUpperCase();
        if (tagName=='SELECT') {
            setCookie(cookieName, ''+tag.selectedIndex, yearHence); // S1062319
        }
        else if (tagName=='INPUT') {
            setCookie(cookieName, (tag.checked ? 'TRUE' : 'FALSE'), yearHence); // S1062319
        }
    }
}


// Show or hide rules based on whether Test+Prod or ProdOnly rules were requested
var multiVersionBaseRuleIds;
function showHideRowsPerRuleState()
{
  // Nothing to do if not creating an estimation
  var excludeTestingRules = document.getElementById("excludeTestingRules");
  if (!excludeTestingRules)
     return;

  // Clear any highlighted rows before switch
  unhighlightAllRows();

  // Populate table of rules with both Testing and Production versions if necessary
  var trs = document.getElementById("ruleTableBody").getElementsByTagName("TR");
  if (multiVersionBaseRuleIds==undefined)
  {
    multiVersionBaseRuleIds = new Object();
    var baseRuleIds = new Object();
    for (var r=0; r<trs.length; r++)
    {
      var tr = trs[r];
      if (tr.getAttribute("baseruleid"))
      {
        var baseRuleId = tr.getAttribute("baseruleid");
        if (baseRuleIds[baseRuleId])
          multiVersionBaseRuleIds[baseRuleId] = baseRuleIds[baseRuleId] + " " + tr.id;
        else
          baseRuleIds[baseRuleId] = tr.id;
      }
      // log(tr.id+" "+tr.getAttribute("baseruleid")+" "+tr.getAttribute("rulestate"));
    }
  }

  var trCount = 1;
  for (var r=0; r<trs.length; r++)
  {
    var tr = trs[r];
    var display = 'none';
    if (tr.getAttribute("rulestate"))
    {
      if ( (excludeTestingRules.checked && tr.getAttribute("rulestate")=='Production') ||
          (!excludeTestingRules.checked && (tr.getAttribute("rulestate")=='Testing' || !multiVersionBaseRuleIds[tr.getAttribute("baseruleid")])) )
      {
        display='';
        trCount++;
      }
      tr.style.display = display;
      tr.className = 'dataRow' + (trCount%2+1) + '_tableRow';
      if (multiVersionBaseRuleIds[tr.getAttribute("baseruleid")])
        document.getElementById('state_'+tr.id).style.fontWeight= 'bold';
    }
  }
}




// for single select
var selectedEstimateId = '';

// for multi select
var selectedEstimateIdList = [];
var selectedEstimateIds = '';

function handleEstimateRowSelection(id, name, row, className, event)
{
   var estimate = new Object();
   estimate.id = id;
   estimate.name = name;
   highlightRow(row, className, event, estimate);
}

function setSelectedEstimate(id, name) {
	selectedEstimateId = id;
}

function setSelectedEstimates() {
	selectedEstimateId = "";
	selectedEstimateIds = "";
	// iterate through keys in hashmap
	for(var i in _highlightedRowMap) {
		var selection = _highlightedRowMap[i];
		if((selection != null) && isDefined(selection)) {
			var estimate = selection.mappedObject;
			selectedEstimateIds += estimate.id + ",";
		}
	}
	// for multi select, trim out the extra ,
	if(selectedEstimateIds.length > 0) {
		selectedEstimateIds = selectedEstimateIds.substring(0, selectedEstimateIds.length -1);
	}
	selectedEstimateIdList = (selectedEstimateIds === "") ? [] : selectedEstimateIds.split(","); // S1441969
	// for single select
	if(selectedEstimateIdList.length === 1) {
       selectedEstimateId = selectedEstimateIdList[0];
	}
}

function newEstimation() {
	var selectedNodes = top.frames['overview']._rulesFolderTree.getSelectedNodes();
	var node = selectedNodes[0];
	var nodeId = node.getId();
	var target = top.frames['overview'].ruleDetailAction + '?method=listRules&showAll=true&workflow=' + encodeURIComponent(nodeId).replace("'","&apos;");
	parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimation");
    parent.frames[ "detail" ].location.href = target;
}

function showEstimateReport() {
    setSelectedEstimates();
   _showEstimateReport(selectedEstimateId);
}

function _showEstimateReport(selectedEstimateId, selectedUser, reportId) {
    if (selectedEstimateId == "") {
        alert($i18n("rules.ruleEstimate.js.selectRow.alert.txt"));
       return;
    }
    if (isEstimateFromForeignBusinessUnit(selectedEstimateId, "viewed")) {
    	return;
    }
    var displayListTag = document.getElementById('displayList');
    var displayList = displayListTag ? displayListTag.value : "";
    if (reportId && reportId!='') {
      if (reportId=='ALL') {
        displayList = 'trxRows alertRows selectedRules trxArdRows keyArdRows trxQueRows keyQueRows';
        displayListTag.value = displayList;
      }
      else {
        if (displayList.indexOf(reportId)<0) {
            displayList += ' ' + reportId;
            displayListTag.value = displayList;
        }
      }
    }
    setBusyDetail();
    parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimatesReport", selectedEstimateId);
    var target = estimateAction + "?method=viewEstimateReport&estimateId=" + selectedEstimateId + "&displayList=" + displayList;
    parent.frames[ "detail" ].location.href = target;
}

function hideEstimateReport(reportId)
{
  var displayListTag = document.getElementById('displayList');
  var displayList = displayListTag.value;
  displayList = displayList.replace(reportId,'');
  displayList = displayList.replace('ALL','');
  displayListTag.value = displayList;
  var tag = document.getElementById(reportId+'_table');
  if (tag) tag.style.display = 'none';
  tag = document.getElementById(reportId+'_closer');
  if (tag) tag.style.display = 'none';
  tag = document.getElementById(reportId+'_opener');
  if (tag) tag.style.display = 'inline';
}

function revealEstimateReport(reportId)
{
  var displayList = document.getElementById('displayList');
  if (displayList.value.indexOf(reportId)<0)
      displayList.value = displayList.value + ' ' + reportId;
  var tag = document.getElementById(reportId+'_table');
     if (tag) tag.style.display = 'inline';
  tag = document.getElementById(reportId+'_closer');
     if (tag) tag.style.display = 'inline';
  tag = document.getElementById(reportId+'_opener');
     if (tag) tag.style.display = 'none';
}

function showDetailReport(cellId,totalRows,whereClause,popWindow,ignoreTotalRows)
{
  var estimateId = parent.frames['detail'].getEstimateId();
  var displayListTag = document.getElementById('displayList');
  var displayList = displayListTag ? displayListTag.value : '';
  var url;
  if (totalRows<=0) {
    alert($i18n("rules.ruleEstimate.js.noRows.alert.txt"));
    return;
  }
  if (ignoreTotalRows=='true')
      totalRows = '';
  if (whereClause)
      whereClause = '&whereClause=' + encodeURIComponent(whereClause);
  else
      whereClause='';
  if (cellId==0)
    url = estimateAction + '?method=viewEstimateDetailReport&tableType=estimationTransactions&totalRows=' + totalRows + '&estimateId=' + estimateId + '&displayList=' + displayList + whereClause;
  else if (cellId=='H')
    url = estimateAction + '?method=viewEstimateDetailReport&tableType=highruleview&totalRows=' + totalRows + '&estimateId=' + estimateId + '&displayList=' + displayList + whereClause;
  else if (cellId=='N')
    url = estimateAction + '?method=viewEstimateDetailReport&tableType=notetranview&totalRows=' + totalRows + '&estimateId=' + estimateId + '&displayList=' + displayList + whereClause;
  else if (cellId=='L')
    url = estimateAction + '?method=viewEstimateDetailReport&tableType=highlinkview&totalRows=' + totalRows + '&estimateId=' + estimateId + '&displayList=' + displayList + whereClause;
  else
  {
    var cellParts = cellId.split('-');
    var fireTypeName = cellParts[0];
    var ruleIdVer = cellParts[1];
    var fraudTypeCode = cellParts[2];
    var column = cellParts[3];
    var alertTypeName = totalRows==column ? '' : totalRows.substring(0,totalRows.length-column.length-1);
    cellId = fireTypeName +'-'+ ruleIdVer +'-'+ fraudTypeCode +'-'+ totalRows;

    // If reference cell exists, use its value for totalRows count
    var mtTag = document.getElementById(cellId);
    if (mtTag)
    {
      totalRows = mtTag.innerHTML.replace(/^\s*|\s*$/g,"");
      if (totalRows<=0)
      {
        alert($i18n("rules.ruleEstimate.js.noTransactions.alert.txt"));
        return;
      }
    }
    else
       totalRows = '';

    if (!fireTypeName)
      fireTypeName='';
    if (!column)
      column = '';
    if (!ruleIdVer)
      ruleIdVer='';
    url = estimateAction + '?method=viewEstimateDetailReport' +
          '&ruleIdVer=' + ruleIdVer + '&totalRows=' + totalRows +
          '&estimateId=' + estimateId + '&fraudTypeCode=' + fraudTypeCode +
          '&fireTypeName=' + fireTypeName + '&displayList=' + displayList +
          '&tableType=estimationTransactions' +
          '&column=' + column +
          '&alertTypeName=' + alertTypeName;
  }

  if (popWindow) {
      popWindow.location.href = url + "&popup=true";
  }
  else {
      setBusyDetail();
      parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimatesReport", estimateId);
      parent.frames[ "detail" ].location.href = url;
  }
}

function showAlertsReport(queueId,queueNameTagId,alertType,totalRows,tableType,alertTypeCode)
{
  if (!totalRows)
    totalRows = document.getElementById("totalRows");
  if (!totalRows || totalRows<=0)
  {
    alert($i18n("rules.ruleEstimate.js.noAlerts.alert.txt"));
    return;
  }

  var queueName = encodeURIComponent(document.getElementById(queueNameTagId).innerHTML);

  // Create filter definitions for redistribution table
  var filters = new Array();
  if (tableType=='estimationRedist') {
      if (alertType.indexOf('ADDS')>=0) {
          if (queueId) {
              filters.push('&filterCheck_PROPOSED_ALERT_QUEUE=' + queueName);
              filters.push('&filterCheck_ACTION=MOVE');
              filters.push('&filterCheck_ACTION=R_MOVED');
              filters.push('&filterCheck_ACTION=R_NOMOVE');
          }
          filters.push('&filterCheck_ACTION=CREATE');
          filters.push('&filterCheck_ACTION=R_CREATED');
          filters.push('&filterCheck_ACTION=R_NOCREATE');
          if (alertTypeCode)
              filters.push('&filterCheck_ALERT_TYPE=' + alertTypeCode);
      }
      else if (alertType.indexOf('DELS')>=0) {
          if (queueId) {
              filters.push('&filterCheck_EXISTING_ALERT_QUEUE=' + queueName);
              filters.push('&filterCheck_ACTION=MOVE');
              filters.push('&filterCheck_ACTION=R_MOVED');
              filters.push('&filterCheck_ACTION=R_NOMOVE');
          }
          filters.push('&filterCheck_ACTION=CLOSE');
          filters.push('&filterCheck_ACTION=R_CLOSED');
          filters.push('&filterCheck_ACTION=R_NOCLOSE');
          if (alertTypeCode)
              filters.push('&filterCheck_ALERT_TYPE=' + alertTypeCode);
      }
  }
  var estimateId = parent.frames['detail'].getEstimateId();
  var displayListTag = document.getElementById('displayList');
  var displayList = displayListTag ? displayListTag.value : '';
  var url = estimateAction + '?method=viewAlertDetailReport' +
        '&queueId=' + queueId +
        '&queueName=' + queueName +
        '&totalRows=' + totalRows +
        '&estimateId=' + estimateId +
        '&tableType=' + tableType +
        '&displayList=' + displayList +
        '&alertType=' + alertType +
        filters.join('');
  setBusyDetail();
  parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimatesReport", estimateId);
  parent.frames[ "detail" ].location.href = url;
}

function showSASLog(logNum)
{
    setSelectedEstimates();
    if(selectedEstimateId == '')
    {
        alert($i18n("rules.ruleEstimate.js.selectRow.alert.txt"));
       return;
    }
    _showSASLog(selectedEstimateId, logNum);
}

function _showSASLog(estimateId, logNum)
{
    if (isEstimateFromForeignBusinessUnit(estimateId, "viewed"))
        return;
    if (!logNum)
        logNum = 1;
    target = estimateAction + '?method=viewLog' + '&logNum=' + logNum + '&estimateId=' + estimateId ;
    // setBusyDetail();
    parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimatesReport", estimateId);
    parent.frames[ "detail" ].location.href = target;
}

function showSASCode()
{
    setSelectedEstimates();
    if(selectedEstimateId == '')
    {
        alert($i18n("rules.ruleEstimate.js.selectRow.alert.txt"));
       return;
    }
   _showSASCode(selectedEstimateId);
}

function _showSASCode(estimateId)
{
    if (isEstimateFromForeignBusinessUnit(estimateId, "viewed"))
        return;
    target = estimateAction + '?method=viewCode'
                                             + '&estimateId=' + estimateId ;
    // setBusyDetail();
    parent.sas.fscf.getCurrentDetailToolbar().setContentFor("estimatesReport", estimateId);
    parent.frames[ "detail" ].location.href = target;
}


// Date validation

// Declaring valid date character, minimum year and maximum year
var dtCh= "/";
var minYear=1900;
var maxYear=2100;

function isInteger(s){
   var i;
    for (i = 0; i < s.length; i++){
        // Check that current character is number.
        var c = s.charAt(i);
        if (((c < "0") || (c > "9"))) return false;
    }
    // All characters are numbers.
    return true;
}

function stripCharsInBag(s, bag){
   var i;
    var returnString = "";
    // Search through string's characters one by one.
    // If character is not in bag, append to returnString.
    for (i = 0; i < s.length; i++){
        var c = s.charAt(i);
        if (bag.indexOf(c) == -1) returnString += c;
    }
    return returnString;
}

function daysInFebruary (year){
   // February has 29 days in any year evenly divisible by four,
    // EXCEPT for centurial years which are not also divisible by 400.
    return (((year % 4 == 0) && ( (!(year % 100 == 0)) || (year % 400 == 0))) ? 29 : 28 );
}
function DaysArray(n) {
   for (var i = 1; i <= n; i++) {
      this[i] = 31
      if (i==4 || i==6 || i==9 || i==11) {this[i] = 30}
      if (i==2) {this[i] = 29}
   }
   return this
}

function isDate(dtStr){
   var daysInMonth = DaysArray(12)
   var pos1=dtStr.indexOf(dtCh)
   var pos2=dtStr.indexOf(dtCh,pos1+1)
   var strMonth=dtStr.substring(0,pos1)
   var strDay=dtStr.substring(pos1+1,pos2)
   var strYear=dtStr.substring(pos2+1)
   strYr=strYear
   if (strDay.charAt(0)=="0" && strDay.length>1) strDay=strDay.substring(1)
   if (strMonth.charAt(0)=="0" && strMonth.length>1) strMonth=strMonth.substring(1)
   for (var i = 1; i <= 3; i++) {
      if (strYr.charAt(0)=="0" && strYr.length>1) strYr=strYr.substring(1)
   }
   month=parseInt(strMonth)
   day=parseInt(strDay)
   year=parseInt(strYr)
   if (pos1==-1 || pos2==-1){
      //alert("The date format should be : mm/dd/yyyy")
      return false
   }
   if (strMonth.length<1 || month<1 || month>12){
      //alert("Please enter a valid month")
      return false
   }
   if (strDay.length<1 || day<1 || day>31 || (month==2 && day>daysInFebruary(year)) || day > daysInMonth[month]){
      //alert("Please enter a valid day")
      return false
   }
   if (strYear.length != 4 || year==0 || year<minYear || year>maxYear){
      //alert("Please enter a valid 4 digit year between "+minYear+" and "+maxYear)
      return false
   }
   if (dtStr.indexOf(dtCh,pos2+1)!=-1 || isInteger(stripCharsInBag(dtStr, dtCh))==false){
      //alert("Please enter a valid date")
      return false
   }
return true
}

// Return whether a string is a time 00:00:00 - 23:59:59 (optional leading zeros, optional seconds)
function isTime(tmStr){
   tmStr = trim(tmStr);
   var isHHMM = tmStr.match(/^([0-1][0-9]|[2][0-3]|[0-9]):([0-5][0-9]|[0-9])(:([0-5][0-9]|[0-9]))?$/);
   return isHHMM;
}

// Return whether a string is a time 00:00:00 - 24:00:00 (optional leading zeros, optional seconds)
function isTime24(tmStr){
   tmStr = trim(tmStr);
   return isTime(tmStr) || tmStr.match(/^([2][4]):([0][0]?)(:([0][0]?))?$/);
}


// Return whether a string is a datetime
function isDateTime(datimStr)
{
   datimStr = trim(datimStr);
   var parts = datimStr.split(" ");
   var timix = parts.length - 1;
   var verdict = (timix < 1 || parts[timix]=="") ? isDate(parts[0])
                                                 : isDate(parts[0]) && isTime(parts[timix]);
   return verdict;
}

function trim(txt)
{
   return txt.replace(/^\s+/g, '').replace(/\s+$/g, '');
}

// Toggles updateability of no-recontact days with checkbox check
function toggleNoRecontactDays(boxTag)
{
  var dayTag = document.getElementById("noRecontactDays");
  if (boxTag.checked) {
    dayTag.disabled = false;
    dayTag.style.backgroundColor = 'white';
  } else {
    dayTag.disabled = true;
    dayTag.style.backgroundColor = '#EEEEEE';
  }
}

// Toggles updateability of sample selections with checkbox check S0870006
function toggleSampleSelections(boxTag)
{
  var thdTag = document.getElementById("sampleThreads");
  var bktTag = document.getElementById("sampleBucket");
  var pctTag = document.getElementById("samplePercent");
  var keyTag = document.getElementById("sampleKey");
  if (boxTag.checked) {
    keyTag.disabled = false;
    pctTag.disabled = false;
    pctTag.style.backgroundColor = 'white';
    bktTag.disabled = false;
    bktTag.style.backgroundColor = 'white';
    thdTag.disabled = false;
    thdTag.style.backgroundColor = 'white';
  } else {
    keyTag.disabled = true;
    pctTag.disabled = true;
    pctTag.style.backgroundColor = '#EEEEEE';
    bktTag.disabled = true;
    bktTag.style.backgroundColor = '#EEEEEE';
    thdTag.disabled = true;
    thdTag.style.backgroundColor = '#EEEEEE';
  }
}

// Does processing needed upon completion of page load for estimate screens
function estimatesLoadHandler()
{
  removeBusy();

  // Establish the onResize handler (complain immediately if someone else is using it)
  //if (window.onresize && window.onresize != rulesTabResizeHandler)
    //alert($i18n("rules.ruleEstimate.js.notResizeHandler.alert.txt"));
  //else
    //window.onresize = rulesTabResizeHandler;
  //rulesTabResizeHandler();

  // Display active filter (if any)
  updateFilterCorral();

  // Check boxes per instructions (if any)
  playbackActionCommands();
}



// Creates a printable form of estimations
function printEstimations()
{
  // Setup multiple selections
  setSelectedEstimates();
  if (selectedEstimateIds.length == 0)
  {
    alert($i18n("rules.ruleEstimate.js.selectRows.alert.txt"));
    return;
  }
  for (var i=0; i<selectedEstimateIds.split(',').length; i++) {
      if (isEstimateFromForeignBusinessUnit(selectedEstimateIds.split(',')[i], "printed"))
          return;
  }

  var w = window.open("rules/printableEstimations.jsp",
                      'PrintableEstimations'+new Date().valueOf(),
                      'menubar,toolbar,resizable=1, scrollbars=1, status=1, width=800, height=600');
  w.focus();
}


// Drill to popup details on highlights/notes
function drillToEstimationPopup(col, count, tableType, whereTemplate, whereArgument1, whereArgument2) {
    if (count<=0) {
        alert($i18n("rules.ruleEstimate.js.nothing.alert.fmt.txt", [col]));
    }
    else {
        var whereClause = whereTemplate.replace('#',whereArgument1);
        whereClause = ' where ' + whereClause.replace('#',whereArgument2);
        var popName = "EstimationResultsPopup";
        var popWindow = open("",popName,'resizable=1,scrollbars=1,status=1,width=650,height=450,left=' + (parent.screenLeft+200) + ',top=' + (parent.screenTop+65));
        popWindow.focus();
        showDetailReport('L',count,whereClause,popWindow);
    }
}

function showAdvanced() {
   document.getElementById('advancedTuningOptions').style.display = '';
   document.getElementById('showAdvancedButton').style.display = 'none';
}

function hideAdvanced()
{
   document.getElementById('advancedTuningOptions').style.display = 'none';
   document.getElementById('showAdvancedButton').style.display = '';
}

// If the indicated estimate is from a non-matching business unit, we may not view, edit, or otherwise access it
function isEstimateFromForeignBusinessUnit(id, operation) {
    if (parent._estimateIdsFromForeignBusinessUnits[id] == true) {
        alert($i18n("rules.ruleEstimate.js.estimationFromOtherBU.alert.fmt.txt", [id, operation]));
        return true;
    }
    return false;
}

function listEstimations() {
	showRuleEstimates(top.frames['overview'].estimateDetailAction);
}

function isAllEstimationsSelected() {
	const ALL_ESTIMATIONS_ID = "ESTIMATION_USER_allEstimates";
	const ESTIMATIONS_ID = "Estimations";
	
	let selectedNodes = top.frames["overview"]._rulesFolderTree.getSelectedNodes();
	let selectedNodeId = (selectedNodes.length === 0) ? ALL_ESTIMATIONS_ID : selectedNodes[0].getId();
	
	return [ALL_ESTIMATIONS_ID, ESTIMATIONS_ID].indexOf(selectedNodeId) != -1;
}

