#/bin/bash
# Script that runs the cpp2sds sas program to create SAS
# datasets from the CPP output files
# The script needs to be executed using the batch user 
# where the follwoing environment variables are defined
# SAS_ROOT                 where sas executable resides
# SASFMCP_IMPLEMENT_HOME   location of the batch server
# SAS_MODELS_HOME          location of the model - needed for mdchash location
# The value can be found in $SASFRAUD_HOME/$IMPLEMENT_FOLDER/engine/Server1/etc/ose.properties
# if the OnDemand Scoring Engine is configured on the same host as the Batch Server
# or in $SASOSE_IMPLEMENT_HOME/Server1/etc/ose.properties on a host where the
# OnDemand Scoring Engine is configured

# check environment variables
# identify the latest cardlist file created by the CPP
# invoke the cpp2sds SAS program to populate $SAS_MODELS_HOME/mdchash
# For now it is assumed that cpp2sds.sas resides in the same directory where
# this script resides
# The YYYYMMDD_HHMM_cpp2sds.log is created in teh cpp logs directory
 
#set -x

readonly acronym=ccca
sas_pgm=cpp2sds.sas

######################################################
# Check the presence of required environment variables 
######################################################
function check_env() {

  rc=0
  if [[ -z "${SAS_ROOT}" ]]; then
    echo SAS_ROOT variable is not defined.
    rc=$((rc+1))
  fi  

  if [[ -z "${SASFRAUD_HOME}" ]]; then
    echo SASFRAUD_HOME variable is not defined.
    rc=$((rc+1))
  fi  
 
  if [[ -z "${IMPLEMENT_FOLDER}" ]]; then
    echo IMPLEMENT_FOLDER variable is not defined.
    rc=$((rc+1))
  fi  

  if [[ -z "${SASFMCP_IMPLEMENT_HOME}" ]]; then
    echo SASFMCP_IMPLEMENT_HOME variable is not defined.
    rc=$((rc+1))
  fi 

  if [[ -z "${SAS_MODELS_HOME}" ]]; then
    echo SAS_MODELS_HOME variable is not defined. 
    rc=$((rc+1))
  fi

  if [[ $rc -gt 0 ]]; then
    echo "rc=" $rc exiting
    exit $rc
  fi

  NOWD=$(date +"%Y%m%d")
  NOWT=$(date +"%H%M")
  cpp2sds_log=${NOWD}_${NOWT}_cpp2sds.log

}

#####################################################################
# Identify CPP_OUTDIR location - where the cardlist files are located
# Identify the lates file created by the CPP job
#
#####################################################################
function get_cpp_dir() {

  cpp_etc_dir=${SASFMCP_IMPLEMENT_HOME}/cpp/etc

  cpp_xml_file=$(ls ${SASFMCP_IMPLEMENT_HOME}/cpp/etc/cpp_${acronym}_jobparms.xml )
    
  echo cpp_xml_file="${cpp_xml_file}"
  if [[ -z "$cpp_xml_file" ]]; then
    echo "Missing cpp_${acronym}_jobparms.xml file at ${cpp_etc_dir}"    
  fi

# identify cpp_outdir location
  cpp_outdir_xml_entry=$( grep "CPP_OUTDIR" ${cpp_xml_file} )

  cpp_outdir_entry=$(sed -e 's/<[^>]*>//g' <<< ${cpp_outdir_xml_entry})

# if the cpp_output_dir contains "&amp;IMPLEMENT_ROOT", perform substitution

  if [ `expr index "&amp;IMPLEMENT_ROOT" ${cpp_outdir_entry}` -ge 1 ]; then
    z=${cpp_outdir_entry#&amp;IMPLEMENT_ROOT}
    cpp_outdir="${SASFMCP_IMPLEMENT_HOME}${z}"
  else 
    cpp_outdir=${cpp_outdir_entry}
  fi
    
# identify cards at risk file pattern
  ent_cards_at_risk_xml_entry=$( grep "ENT_CARDS_AT_RISK" $cpp_xml_file )
  ent_cards_at_risk=$(sed -e 's/<[^>]*>//g' <<< $ent_cards_at_risk_xml_entry)

# identify the latest file created by the CPP job
  cpp_file_prefix="$( sed -e s/"&amp".*// <<< ${ent_cards_at_risk})"
}

function get_latest_cpp_file {

# guard against Windows files: eliminate cr
  cpp_outdir=$(echo ${cpp_outdir//[$'\r']})
  cpp_file_prefix=$(echo ${cpp_file_prefix//[$'\r']})
  
  
#
  latest_cpp_file=$(ls -t ${cpp_outdir}/${cpp_file_prefix}* | head -1 )
  export CPP_CARD_FILE=$latest_cpp_file
  env | grep CPP

}
######################################################
#
######################################################
function invoke_cpp2sds() {
  NOWD=$(date +"%Y%m%d")
  NOWT=$(date +"%H%M")
  cpp2sds_log=${NOWD}_${NOWT}_cpp2sds.log
  sas $SASFMCP_IMPLEMENT_HOME/cpp/src/sas/${sas_pgm} -log $SASFMCP_IMPLEMENT_HOME/cpp/logs/${cpp2sds_log}
}

check_env

get_cpp_dir

get_latest_cpp_file

invoke_cpp2sds
