/*
 * Highly specific macro for driving evaluation and report of quarterly reports
 * 
 * Author: Song, Qiang <qiang.song@sas.com> 2015
 * 
 */

/*
 * Assuming &report_root, &TSTART, &TEND are defined 
 */

%macro evaluate_and_report(trans = , portid = );

    %let reportfile = &report_root./report_output/quarterly/&dev_data_acronym/QUARTERLY_&TEND._&portid..pdf;

    %if %lowcase(&type) ne dev_quarterly %then %do;
        %let evalout = qtrlib.eval_&TSTART._&TEND;
        %let statout = qtrlib.stat_&TSTART._&TEND;
    %end;
    %else %do;
        %let evalout = qtrlib.eval_dev_&TSTART._&TEND;
        %let statout = qtrlib.stat_dev_&TSTART._&TEND;
    %end;    

    %global num_input_trans;
    %numobs(data=&trans, macrovar=num_input_trans);
    
    %if &num_input_trans = 0 %then %do;
        %put portfolio: &portid;
        %zero_input;
    %end;
    %else %do;    

        %get_population_statistics(in = &trans, out = &statout);

        proc sql noprint;
            select normalized_case_frd_rate into :normalized_case_frd_rate from &statout;
        quit;    
        
        %if &normalized_case_frd_rate ne 0 %then %do;
            %run_evaluation_mac(
                in = &trans, out = &evalout,
                rh_sampling_rate = &rh_sampling_rate,
                rh_sampling_rate_fraud = &rh_sampling_rate_fraud);

            %if %lowcase(&type) ne dev_quarterly %then %do;
                %final_reports_quarterly(qtrlib = qtrlib, reportfile = &reportfile, portid = &portid);
            %end;    
        %end;
        %else %do;
            %put ================================================================;
            %put portfolio: &portid;
            %put No fraudulent transactions are reported in current evaluation period;
            %put Model performance report is disabled;
            %put ================================================================;
        %end;    

    %end;

    libname qtrlib clear;
%mend;    
