%macro final_reports_daily_monthly;

ods listing close;

%if %in1(&TYPE,DAILY MONTHLY) AND &tablecount>=1 %then %do;

proc sort data=&config_ds;
 by seq_original;
run;

*--------------------------------------------------------------------------*
|  Code to Modify Config File is Below                                     |
*--------------------------------------------------------------------------*;

%add_nobs_to_config2;

*--------------------------------------------------------------------------*
|  Code to Modifying Config File is Above                                  |
*--------------------------------------------------------------------------*;


**** macro variable type_long is used in titles ****;

      %if %in1(&type,DEV_DAILY DEV_MONTHLY) %then %let type_long=Development;
%else %if &TYPE eq DAILY %then %let type_long=Daily;
%else %if &TYPE eq MONTHLY %then %let type_long=Monthly;


*--------------------------------------------------------------------------*
| Add Labels to Parameter Dataset for Final Processing                     |
*--------------------------------------------------------------------------*;

proc sort data=date_range out=check_range(keep=label);
 by label;
 run;

 proc transpose data=check_range out=dates_trans(drop=_name_ _label_)prefix=time_period_;
 var label;
 run;

 *--------------------------------------------------------------------------*
| Define Macro Variables to be used for Reporting                          |
*--------------------------------------------------------------------------*;

 proc sql;
  select compbl(label)
  into :time_period_1-:time_period_%left(&time_periods)
  from check_range;

 %put time_period_1: &time_period_1;
 %put time_period_2: &time_period_2;

data _temp_config3;
 set _temp_config2 end=last;
 by report_variable drill_down_variable NOTSORTED;

     if _n_=1 then set dates_trans;

    /** the following needed for card-level variables **/

    aggregation_type=upcase(aggregation_type);
    length drill_number 8;
    drill_number=scan(drill_vname,3,"_");


	frh_table=strip("&frh_table");

	length report_variable_shortname $27;
	report_variable_shortname=upcase(strip(substr(report_variable,1,27)));

         if type in('DEV_DAILY','DEV_MONTHLY') then outputds_prefix='Dev    ';
    else if type='DAILY'   then outputds_prefix='Daily  ';
    else if type='MONTHLY' then outputds_prefix='Monthly';

   title_rep_var_lab=translate(Report_Variable_Label," ","|");  *** need to remove split character from title ***;

    /*** the following header information - contains the title, pdf file, ods escape char, and title - should be the same for T and C level Reports ***/

             X='if _break_ ne " " then call define("'||strip(Report_Variable)||'","style","style=[pretext='||"'Total:']"||'");';
             *** coding the above as a character - the combinations of quotes get tricky within call execute - uses for T-level - but levels no longer exist here;
            title=strip("&title"); ***&title will not resolve in single quotes within call execute and title statement;
            title_rep_var_lab=translate(Report_Variable_Label," ","|");  *** need to remove split character from title ***;
    if upcase(stability_index) in('P','D') then do;
          stab_index_col_var="stability_index";
          cindex1=&cindex1;
          if upcase(stability_index)='P' then do;
	           if type ='DAILY' then stab_period="Previous Day";
               else if type='MONTHLY' then stab_period="Previous Month";
             cindex2=&cindex2_P; end;
          else if upcase(stability_index)='D' then do;
             stab_period="Development";
             cindex2=&cindex2_D;
          end;
    end;
    else stab_index_col_var=" ";

    if aggregation_result_format=' ' then aggregation_result_format="comma15.";
run;

 proc sql noprint;
 select development_period
 into :development_period
 from _temp_config(obs=1);

 %put development_period: &development_period;

*--------------------------------------------------------------------------*
| Parameter Data Set to Generate Final Set of Reports                      |
*--------------------------------------------------------------------------*;

data _null_;
 set _temp_config3 end=last;
 by report_variable drill_down_variable NOTSORTED;


where DRILL_NUMBER NE .;  **** ONLY PROCESS DRILL DOWN REPORTS IF DRILL_NUMBER PRESENT *******;
RUN='DRILL';   *** instruction for fixnumeric and fixchar macros ****;
plot_data="plot_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number);
output_qa_data=upcase("&output_qa_data");

call execute('%let type = ' || type || ';');
call execute('%let varcount = ' || varcount || ';');
call execute('%let drill_number = ' || drill_number || ';');
call execute('%let report_variable = ' || report_variable || ';');
call execute('%let report_variable_format = ' || report_variable_format || ';');
call execute('%let drill_down_format = ' || drill_down_format || ';');
call execute('%let drill_down_label = ' ||strip(drill_down_label)||';');

*--------------------------------------------------------------*
|  Merge together Dev and Daily/Monthly Output Datasets from   |
|  Proc Report for final Reports                               |
*--------------------------------------------------------------*;
call execute('
proc sql;
create table final_report_&type._var_&varcount._dv_&drill_number. as
  select
    coalesce(one._report_var_1, two._report_var_2) as &report_variable format = &report_variable_format,
    coalesce(one._drill_var_1, two._drill_var_2) as drill_var_&drill_number  label="&drill_down_label" /**format = &drill_down_format**/,
    /*** hotfix in comment above - see note below ***/
    one.*,
    two.*,
    1 as holder
  from
    work.&type._report_var_&varcount._dv_&drill_number
           (rename = (&report_variable = _report_var_1 drill_var_&drill_number = _drill_var_1))
    as one
    full join
    rep.dev_report_var_&varcount._dv_&drill_number
           (rename = (&report_variable =  _report_var_2 drill_var_&drill_number = _drill_var_2))
    as two
  on
     put(one._report_var_1, &report_variable_format) = put(two._report_var_2, &report_variable_format.)
     and
     /***put(one._drill_var_1, &drill_down_format) = put(two._drill_var_2, &drill_down_format)****/
     /*** replacing the code above is a HOTFIX - formats for drill down should already be applied by this step ***/
     /*** applying the format twice at this step generally creates bizzare results ****/
     one._drill_var_1=two._drill_var_2
  order by
    drill_var_&drill_number, &report_variable;
quit;
');

/*********** the following is drill version ***************/

call execute("ods escapechar='~';");
call execute("ods pdf body='&report_path"||strip(Type)||"_"||strip(frh_table)||"_&report_date9._"||strip(Report_Variable)||"_Drill_BY_"||strip(Drill_Down_variable)||".pdf'  pdftoc=2 contents=NO style=&drillstyle;");
call execute("ods proclabel '"||strip(report_variable)||": distribution report';");

call execute('title1 "~S={foreground=green}'||TITLE||" &type_long "||strip(frh_table)||" - &report_date9. - "||strip(title_rep_var_lab)||" "||strip(Report_Variable)||'";');
call execute("title2;");

call execute("proc report data=final_report_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" split='|' missing completecols  nowindows contents='distribution report' ");
if output_qa_data='Y' then call execute("out=qalib."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||"_dv_"||strip(drill_number)||";");
call execute(";");
call execute("   by  drill_var_"||strip(drill_number)||";");

if upcase(type)='DAILY' then do;  *** for Now - Daily has 3 time periods defined and Monthly has 2 - this could change ***;
  call execute("columns  "||strip(report_variable)||" ('"||strip(time_period_1)||"' _C2_=C2 _C3_=C3) ('"||strip(time_period_2)||"' _C4_=C4 _C5_=C5) ");
  call execute("('"||strip(time_period_3)||"' _C6_=C6 _C7_=C7)");
   if upcase(stability_index) in('P','D') then call execute("('"||strip(development_period)||"' _D1_ _D2_) Stability_Index holder;");
   else call execute("('"||strip(development_period)||"' _D1_ _D2_)  holder;");
  call execute("define C6/  style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
  call execute("define C7/ 'Percent' style(column)=[cellwidth=&cellwidth] format=percent10.2;");
end;

else if upcase(type)='MONTHLY' then do;  *** for Now - Daily has 3 time periods defined and Monthly has 2 - this could change ***;
  call missing(time_period_3);
  call execute("columns  "||strip(report_variable)||" ('"||strip(time_period_1)||"' _C2_=C2 _C3_=C3) ('"||strip(time_period_2)||"' _C4_=C4 _C5_=C5) ");
   if upcase(stability_index) in('P','D') then call execute("('"||strip(development_period)||"' _D1_ _D2_) Stability_Index holder;");
    else call execute("('"||strip(development_period)||"' _D1_ _D2_)  holder;");
end;

call execute("define holder/noprint order;");
call execute("define "||strip(report_variable)||"/display '"||strip(report_variable_label)||"' style(column)=[just=left] id ;");
call execute("define C2/ style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
call execute("define C3/ style(column)=[cellwidth=&cellwidth] format=percent10.2;");
call execute("define C4/   style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
call execute("define C5/ 'Percent'  style(column)=[cellwidth=&cellwidth] format=percent10.2;");
call execute("define _D1_/ style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||" right;");
call execute("define _D2_/ style(column)=[cellwidth=&cellwidth] format=percent10.2 right;");

if upcase(stability_index) in('P','D') then do;

	IF upcase(type)='DAILY' then
	call execute("define stability_index/Computed 'Stability|Index|Current Day vs. "||strip(stab_period)||"' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");
	else
	call execute("define stability_index/Computed 'Stability|Index|Current Month vs. "||strip(stab_period)||"' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");

	call execute("compute stability_index;");
	IF upcase(stability_index)='P' then call execute("if _c&cindex1._ > 0 and _c&cindex2_P._ > 0 then stability_index=(_c&cindex1._-_c&cindex2_P._)*log(_c&cindex1._/_c&cindex2_P._);");
	                        else call execute("if _c&cindex1._ > 0 and _c&cindex2_D._ > 0 then stability_index=(_c&cindex1._-_c&cindex2_D._)*log(_c&cindex1._/_c&cindex2_D._);");
	call execute("  sumindices+stability_index;");
	call execute("  if _break_='_RBREAK_' then stability_index=sumindices;");
	call execute("  if _break_='_RBREAK_' then call define ('Stability_index','style','Style=[backgroundcolor=traffic. color=black]');");
	call execute("endcomp;");
END;

call execute("rbreak after/summarize  style=[background=grey foreground=yellow];");

call execute("compute "||strip(Report_Variable)||";");
call execute(x); *** see character variable definition above;
call execute(" endcomp;");

call execute("break before holder/page contents='';");
call execute("run;");

*----------------------------------------------------------------------------*
|  Percentile Reports For Drill Down Downs                                   |
*----------------------------------------------------------------------------*;
if substr(report_variable_format,1,1) NE '$' and upcase(percentile_report)='Y'  then do;
		*--------------------------------------------------------------*
		|  Process Output Dev Data - Could Save this and do this once  |
		|  But this step runs lightning fast - leave for now           |
		*--------------------------------------------------------------*;

		call execute("proc sort data=Rep.dev_tab_var_"||strip(varcount)||"_dv_"||strip(drill_number));
		call execute("           out=Work.Temp_dev_tab_Var_"||strip(varcount)||"_dv_"||strip(drill_number)||";");
		call execute(" by drill_var_"||strip(drill_number)||";");
		call execute("run;");

		*--------------------------------------------------------------*
		|  Process Output Daily or Monthly Output from Proc Report     |
		|  The code below                                              |
		*--------------------------------------------------------------*;
		****************** sort the temp data set ******************;

		call execute("proc sort data=work."||strip(type)||"_tab_var_"||strip(varcount)||"_dv_"||strip(drill_number));
		call execute("           out=work.Temp_"||strip(type)||"_tab_var_"||strip(varcount)||"_dv_"||strip(drill_number)||";");
	    call execute(" by drill_var_"||strip(drill_number)||";");
		call execute(" run;");
		*--------------------------------------------------------------*
		|  Merge together Dev and Daily/Monthly Output Datasets from   |
		|  Proc Report for final Reports                               |
		|  For Tab Reports do not use Merge because we do NOT want to  |
		|  Sort by the 'statistic' - use sql or hash                   |
		*--------------------------------------------------------------*;
      	call execute("proc sql;");
		call execute("create table fintab_rep_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" as ");
		call execute("select one._D1_, two.*, 1 as holder ");
		call execute("from  work.Temp_dev_tab_Var_"||strip(varcount)||"_dv_"||strip(drill_number)||" as one,");
		call execute("        work.Temp_"||strip(type)||"_tab_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" as two");
		call execute("where one.drill_var_"||strip(drill_number)||"=two.drill_var_"||strip(drill_number)||" AND ");
		call execute("      one."||strip(report_variable)||"=two."||strip(report_variable)||";");

		*--------------------------------------------------------------*
		|  Final Percentile Report Here                                |
		*--------------------------------------------------------------*;

		call execute("ods proclabel '"||strip(report_variable)||": percentile report';");

		call execute("proc report data=fintab_rep_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" split='|' contents='percentile report' missing completecols nowindows ;");
		call execute("   by  drill_var_"||strip(drill_number)||";");
        if upcase(type)='DAILY' then call execute("columns  "||strip(report_variable)||" _C3_=C3 _C2_=C2 _C1_=C1 _D1_ dummy holder;");
		if upcase(type)='MONTHLY' then call execute("columns  "||strip(report_variable)||"  _C2_=C2 _C1_=C1 _D1_ dummy holder;");
		call execute("define "||strip(report_variable)||"/display '"||strip(title_rep_var_lab)||"' style(column)=[just=left];");
		call execute("define C1/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		call execute("define C2/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		if upcase(type)='DAILY' then call execute("define C3/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		call execute("define _D1_/ style(column)=[cellwidth=1.0in] format=comma15. right;");
        call execute("define dummy/computed noprint;");
		call execute("define holder/noprint order;");
        call execute("compute dummy;");
		call execute("if _c1_ in('N','NMiss') then do;");
		call execute("   call define('_c2_','format','comma15.');");
		call execute("   call define('_c3_','format','comma15.');");
		call execute(" call define('_c4_','format','comma15.');");
		if upcase(type)='DAILY' then call execute(" call define('_c5_','format','comma15.');");
		call execute("end;");
		call execute("else do;");
		call execute(" call define('_c2_','format','10.2');");
		call execute(" call define('_c3_','format','10.2');");
		call execute(" call define('_c4_','format','10.2');");
		if upcase(type)='DAILY' then call execute(" call define('_c5_','format','10.2');");
		call execute("end;");
		call execute("endcomp;");
		call execute("break before holder/page contents='';");
		call execute("where UPCASE("||strip(report_variable)||") ne 'NMISS';");
		call execute(" run;");

END;
*----------------------------------------------------------------------------*
|  Plots are below                                                           |
*----------------------------------------------------------------------------*;

call execute("data plot_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number));
call execute("(keep=group "||strip(report_variable)||" percent drill_var_"||strip(drill_number)||");");
call execute(" set final_report_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" ;");
call execute("format percent percent10.0;");

 %do j=1 %to &time_periods;
call execute("group=strip(resolve('&&time_period_&j'));");
call execute("percent=_C%eval(&j*2+1)_;");
call execute("output;");
 %end;
call execute("group=strip(resolve('&development_period'));");
call execute("percent=_D2_;");
call execute("output;");
call execute("run;");

put 'Plot Data: ' plot_data;

IF NUMBER_OBS_DRILL>=11 THEN DO;
	%cleanup_forplots;

	if substr(report_variable_format,1,1) NE '$' then do;
	%fix_numeric;
	end;

	else do;
	%fix_char;
	end;

%macro hideit;
	call execute("ods proclabel '"||strip(report_variable)||": bar charts';");
	call execute("proc sgpanel data=plot_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" description='bar charts' ;");
	call execute("   by  drill_var_"||strip(drill_number)||";");
	call execute(" panelby group;");
	call execute(" rowaxis discreteorder=data label='Percent';");
	call execute(" format sort_var  TMP_"||strip(report_variable_shortname)||"_. ;");
	call execute(" label sort_var='"||strip(report_variable_label)||"';");
	call execute(" vbar sort_var/response=percent;");
	call execute(" where not(missing("||strip(report_variable)||"));");
	call execute(" run;");
%mend;
END;

ELSE DO;
%macro hideit;
    call execute("ods proclabel '"||strip(report_variable)||": bar charts';");
	call execute("proc sgpanel data=plot_"||strip(type)||"_var_"||strip(varcount)||"_dv_"||strip(drill_number)||" description='bar charts' ;");
	call execute("   by  drill_var_"||strip(drill_number)||";");
	call execute(" panelby group;");
	call execute(" rowaxis discreteorder=data;");
	call execute(" vbarparm category="||strip(report_variable)||" response=percent;");
	call execute(" run;");
%mend;
END;

call execute("ods pdf close;");

RUN;

*********************************************************************************************************************;
*********************************************************************************************************************;
*********************************************************************************************************************;
**** MAIN REPORTS ***************************************************************************************************;
*********************************************************************************************************************;
*********************************************************************************************************************;
*********************************************************************************************************************;
*--------------------------------------------------------------------------*
| Parameter Data Set to Generate Final Set of Reports                      |
*--------------------------------------------------------------------------*;

data _null_;
 set _temp_config3 end=last;
 by report_variable drill_down_variable NOTSORTED;

RUN='MAIN';   *** instruction for fixnumeric and fixchar macros ****;
output_qa_data=upcase("&output_qa_data");
plot_data="plot_"||strip(type)||"_var_"||strip(varcount);


if _n_=1 then do;
        call execute("ods escapechar='~';");
        call execute("ods pdf body='&report_path"||strip(Type)||"_"||strip(frh_table)||"_&report_date9..pdf' contents=NO  pdftoc=1 style=&mainstyle;");
end;

call execute('%let type = ' || type || ';');
call execute('%let varcount = ' || varcount || ';');
call execute('%let drill_number = ' || drill_number || ';');
call execute('%let report_variable = ' || report_variable || ';');
call execute('%let report_variable_format = ' || report_variable_format || ';');
call execute('%let drill_down_format = ' || drill_down_format || ';');

*--------------------------------------------------------------*
|  Set Up Titles for Main Reports                              |
*--------------------------------------------------------------*;

 if FIRST.REPORT_VARIABLE then do;
        drill_level=0;
        call execute('title1 "~S={foreground=green}'||TITLE||" &type_long "||strip(frh_table)||" - &report_date9. - "||strip(title_rep_var_lab)||" "||strip(Report_Variable)||'";');
        call execute("title2;");
 end;


 if drill_down_variable ne " " then do;
                drill_level+1;
                title_number=drill_level+2;
                titlelink='LINK="'||strip(Type)||"_"||strip(frh_table)||"_&report_date9._"||strip(Report_Variable)||"_Drill_BY_"||strip(Drill_Down_variable)||'.pdf" justify=L "Drill by: '||strip(Drill_Down_Label)||'";';
                *** coding the above as a character - the combinations of quotes get tricky within call execute;
                call execute("title"||strip(title_number)||" "||titlelink);
 end;


IF LAST.REPORT_VARIABLE THEN DO;

*--------------------------------------------------------------*
|  Merge together Dev and Daily/Monthly Output Datasets from   |
|  Proc Report for final Reports                               |
*--------------------------------------------------------------*;
call execute('
proc sql;
create table final_report_&type._var_&varcount as
  select
    coalesce(one._report_var_1, two._report_var_2) as &report_variable format = &report_variable_format,
    one.*,
    two.*,
    1 as holder
  from
    work.&type._report_var_&varcount(rename = (&report_variable = _report_var_1))
    as one
    full join
    rep.dev_report_var_&varcount(rename = (&report_variable = _report_var_2))
    as two
  on
    put(one._report_var_1, &report_variable_format) = put(two._report_var_2, &report_variable_format.)
  order by
    &report_variable;
quit;
');

*--------------------------------------------------------------*
|  Execute Proc Report for Main Reports                        |
*--------------------------------------------------------------*;



call execute("ods proclabel '"||strip(report_variable)||": distribution report';"); *** repeat heading for each variable;

call execute("proc report data=final_report_"||strip(type)||"_var_"||strip(varcount)||" split='|' missing completecols  nowindows contents='distribution report'  ");
if output_qa_data='Y' then call execute("out=qalib."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";");
call execute(";");
if upcase(type)='DAILY' then do;  *** for Now - Daily has 3 time periods defined and Monthly has 2 - this could change ***;
  call execute("columns  "||strip(report_variable)||" ('"||strip(time_period_1)||"' _C2_=C2 _C3_=C3) ('"||strip(time_period_2)||"' _C4_=C4 _C5_=C5) ");
  call execute("('"||strip(time_period_3)||"' _C6_=C6 _C7_=C7)");
  if upcase(stability_index) in('P','D') then call execute("('"||strip(development_period)||"' _D1_ _D2_) Stability_Index holder;");
  else call execute("('"||strip(development_period)||"' _D1_ _D2_)  holder;");
  call execute("define C6/  style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
  call execute("define C7/ 'Percent'  style(column)=[cellwidth=&cellwidth] format=percent10.2;");
end;

else if upcase(type)='MONTHLY' then do;  *** for Now - Daily has 3 time periods defined and Monthly has 2 - this could change ***;
  call missing(time_period_3);
  call execute("columns  "||strip(report_variable)||" ('"||strip(time_period_1)||"' _C2_=C2 _C3_=C3) ('"||strip(time_period_2)||"' _C4_=C4 _C5_=C5) ");
  if upcase(stability_index) in('P','D') then call execute("('"||strip(development_period)||"' _D1_ _D2_) Stability_Index holder;");
  else call execute("('"||strip(development_period)||"' _D1_ _D2_)  holder;");
end;

call execute("define holder/noprint order;");
call execute("define "||strip(report_variable)||"/display '"||strip(report_variable_label)||"' style(column)=[just=left] ;");
call execute("define C2/ style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
call execute("define C3/ style(column)=[cellwidth=&cellwidth] format=percent10.2;");
call execute("define C4/   style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||";");
call execute("define C5/ 'Percent' style(column)=[cellwidth=&cellwidth] format=percent10.2;");
call execute("define _D1_/ style(column)=[cellwidth=&cellwidth] format="||strip(aggregation_result_format)||" right;");
call execute("define _D2_/ style(column)=[cellwidth=&cellwidth] format=percent10.2 right;");

if upcase(stability_index) in('P','D') then do;
	IF upcase(type)='DAILY' then
	call execute("define stability_index/Computed 'Stability|Index|Current Day vs. "||strip(stab_period)||"' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");
	else
	call execute("define stability_index/Computed 'Stability|Index|Current Month vs. "||strip(stab_period)||"' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");

	call execute("compute stability_index;");
	IF upcase(stability_index)='P' then call execute(" if _c&cindex1._ > 0 and _c&cindex2_P._ > 0 then stability_index=(_c&cindex1._-_c&cindex2_P._)*log(_c&cindex1._/_c&cindex2_P._);");
	                        else call execute(" if _c&cindex1._ > 0 and _c&cindex2_D._ > 0 then stability_index=(_c&cindex1._-_c&cindex2_D._)*log(_c&cindex1._/_c&cindex2_D._);");
	call execute("  sumindices+stability_index;");
	call execute("  if _break_='_RBREAK_' then stability_index=sumindices;");
	call execute("  if _break_='_RBREAK_' then call define ('Stability_index','style','Style=[backgroundcolor=traffic. color=black]');");
	call execute("endcomp;");
end;

call execute("rbreak after/summarize  style=[background=grey foreground=yellow];");

call execute("compute "||strip(Report_Variable)||";");
call execute(x); *** see character variable definition above;
call execute(" endcomp;");
call execute("break before holder/page contents='';");
call execute("run;");

*----------------------------------------------------------------------------*
|  Percentile Reports For Main Reports                                       |
*----------------------------------------------------------------------------*;
if substr(report_variable_format,1,1) NE '$' and upcase(percentile_report)='Y'  then do;

		*--------------------------------------------------------------*
		|  Merge together Dev and Daily/Monthly Output Datasets from   |
		|  Proc Report for final Reports                               |
		|  For Tab Reports do not use Merge because we do NOT want to  |
		|  Sort by the 'statistic' - use sql or hash                   |
		*--------------------------------------------------------------*;
      	call execute("proc sql;");
		call execute("create table fintab_rep_"||strip(type)||"_var_"||strip(varcount)||" as ");
		call execute("select one._D1_, two.*, 1 as holder ");
		call execute("from  Rep.dev_tab_var_"||strip(varcount)||" as one,");
		call execute("        work."||strip(type)||"_tab_var_"||strip(varcount)||" as two");
		call execute("where one."||strip(report_variable)||"=two."||strip(report_variable)||";");

		*--------------------------------------------------------------*
		|  Final Percentile Report Here                                |
		*--------------------------------------------------------------*;

	     call execute("ods proclabel '"||strip(report_variable)||": percentile report';"); *** repeat heading for each variable;

		call execute("proc report data=fintab_rep_"||strip(type)||"_var_"||strip(varcount)||" split='|' missing completecols nowindows contents='percentile report' ; ");
        if upcase(type)='DAILY' then call execute("columns  "||strip(report_variable)||" _C3_=C3 _C2_=C2 _C1_=C1 _D1_ dummy holder;");
		if upcase(type)='MONTHLY' then call execute("columns  "||strip(report_variable)||"  _C2_=C2 _C1_=C1 _D1_ dummy holder;");
		call execute("define "||strip(report_variable)||"/display '"||strip(title_rep_var_lab)||"' style(column)=[just=left];");
		call execute("define C1/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		call execute("define C2/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		if upcase(type)='DAILY' then call execute("define C3/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		call execute("define _D1_/ style(column)=[cellwidth=1.0in] format=comma15. right;");
		call execute("define holder/noprint order;");
        call execute("define dummy/computed noprint;");
        call execute("compute dummy;");
		call execute("if _c1_ in('N','NMiss') then do;");
		call execute("   call define('_c2_','format','comma15.');");
		call execute("   call define('_c3_','format','comma15.');");
		call execute(" call define('_c4_','format','comma15.');");
		if upcase(type)='DAILY' then call execute(" call define('_c5_','format','comma15.');");
		call execute("end;");
		call execute("else do;");
		call execute(" call define('_c2_','format','10.2');");
		call execute(" call define('_c3_','format','10.2');");
		call execute(" call define('_c4_','format','10.2');");
		if upcase(type)='DAILY' then call execute(" call define('_c5_','format','10.2');");
		call execute("end;");
		call execute("endcomp;");
		call execute("break before holder/page contents='';");
		call execute("where UPCASE("||strip(report_variable)||") ne 'NMISS';");
		call execute(" run;");

END;

*----------------------------------------------------------------------------*
|  Plots are below                                                           |
*----------------------------------------------------------------------------*;

**** clear all potential 'drill-down' titles ****;

%do j=3 %to 9;
call execute("title&j;");
%end;

call execute("data plot_"||strip(type)||"_var_"||strip(varcount)||"(keep=group "||strip(report_variable)||" percent);");
call execute(" set final_report_"||strip(type)||"_var_"||strip(varcount)||";");
call execute("format percent percent10.0;");

 %do j=1 %to &time_periods;
call execute("group=strip(resolve('&&time_period_&j'));");
call execute("percent=_C%eval(&j*2+1)_;");
call execute("output;");
 %end;
call execute("group=strip(resolve('&development_period'));");
call execute("percent=_D2_;");
call execute("output;");
call execute("run;");

call execute("data plot_"||strip(type)||"_var_"||strip(varcount)||";");
call execute(" set plot_"||strip(type)||"_var_"||strip(varcount)||" END=LAST NOBS=NUMOBS;");
call execute("run;");

put 'Plot Data: ' plot_data;

IF NUMBER_OBS_MAIN>=11 THEN DO;
	%cleanup_forplots;

	if substr(report_variable_format,1,1) NE '$' then do;
	%fix_numeric;
	end;

	else do;
	%fix_char;
	end;
%macro hideit;
	call execute("ods proclabel '"||strip(report_variable)||": bar chart';"); *** repeat heading for each variable;
	call execute("proc sgpanel data=plot_"||strip(type)||"_var_"||strip(varcount)||" description='bar charts';");
	call execute(" panelby group;");
	call execute(" rowaxis discreteorder=data label='Percent';");
	call execute(" format sort_var  TMP_"||strip(report_variable_shortname)||"_. ;");
	call execute(" label sort_var='"||strip(report_variable_label)||"';");
	call execute(" vbar sort_var/response=percent;");
	call execute(" where not(missing("||strip(report_variable)||"));");
	call execute("run;");
%mend;
END;
ELSE DO;
%macro hideit;
    call execute("ods proclabel '"||strip(report_variable)||": bar charts';"); *** repeat heading for each variable;
	call execute("proc sgpanel data=plot_"||strip(type)||"_var_"||strip(varcount)||" description='bar charts';");
	call execute(" panelby group;");
	call execute(" rowaxis discreteorder=data;");
	call execute(" vbarparm category="||strip(report_variable)||" response=percent;");
	call execute(" run;");
%mend;	
END;

END; **** END OF LOOP FOR MAIN REPORT - LAST.VARIABLE ****;

IF LAST THEN call execute("ods pdf close;");


RUN;
%end;


%mend final_reports_daily_monthly;
