/*
 * extract FRH_Table information
 * 
 * Author: Song, Qiang <qiang.song@sas.com> 2014
 * 
 */

%macro rptcfg_extract_frh_table_info(xmlfile =, frh_table_info =, type =  );
/* DOCSSTRING
This macro extract the list necessary API variables to run analytic reports

#input
- xmlfile: path to the location of the XML configuration file; do not include quotations

#output
- varlst: a SAS dataset containing the list of API variables
    END DOCSTRING */

    %local house_keeping_list;
    %let house_keeping_list = rqo_tran_date rqo_tran_time rqo_proc_utc_datetime hqo_sas_card_rand_dig;

    data _table_var_list_(keep = frh_table var_name);

        if _N_ = 1 then do;
            length
                drl_name                  $32
                drl_var_depending         $1200
                ;
            declare hash drills(dataset: "rptcfg.drill_down_list");
            drills.definekey("drl_name");
            drills.definedata("drl_var_depending");
            drills.definedone();
            
            call missing(of drl_:);
            end;

        set rptcfg.reports(where = (type in  ("all", "&type")));

        length var_name_list  $1200;
        
        if missing(report_variable_depending) then
            var_name_list = catx(" ", var_name_list, report_variable);
        else
            var_name_list = catx(" ", var_name_list, report_variable_depending);

        if not missing(drill_down_list) then do; 
            i = 1;
            drl_name = scan(drill_down_list, i);
            do while (not missing(drl_name));
                rc = drills.find();
                if rc = 0 then do;
                    if not missing(drl_var_depending) then
                        var_name_list = catx(" ", var_name_list, drl_var_depending);
                    else
                        var_name_list = catx(" ", var_name_list, drl_name);
                    end;
                else do;
                    putlog "ERROR: missing drill_down tyle specification for " drl_name;
                    end;
                i = i + 1;
                drl_name = scan(drill_down_list, i); 
                end;
            end;

        length var_name $32;
        i = 1;
        var_name = scan(var_name_list, i);
        do while (not missing(var_name));
            output;
            i = i + 1;
            var_name = scan(var_name_list, i); 
        end;
    run;

    proc sort nodupkey data = _table_var_list_;
        by frh_table var_name;
    run;

    data &frh_table_info(keep = frh_table var_list);
        set _table_var_list_;
        by frh_table;
        
        length var_list $ 32767;
        retain var_list;
        if first.frh_table then
            call missing(var_list);

        var_list = catx(" ", var_list, var_name);

        if last.frh_table then do;
            var_list = lowcase(compbl("&house_keeping_list " || var_list)); 
            output;
            end;
    run;

    proc datasets nolist library = work;
        delete _table_var_list_;
    quit;
        
%mend rptcfg_extract_frh_table_info;

     
