%macro setup;

    %global
        input_ds
        config_ds
        output_recode_ds
        ;
    %let input_ds = &input_dataset;
    filename recode "&report_root./src/recode_&frh_table._&type_freq..sas";

    libname rep "&report_root./data/development_data/&type_freq/&frh_table"; 

    %if %upcase(&output_qa_data) = Y %then %do; 
        X "mkdir -p &report_root./qa_data/&type_freq./";
        libname qalib "&report_root./qa_data/&type_freq./";
    %end;

    %let config_ds = work.config_&frh_table._&type_freq;
    data &config_ds;
        set &config_dataset(where = (frh_table = "&frh_table"));
    run;    
    
    %let output_recode_ds=work.&frh_table._recode;


*-----------------------------------------------------------------------*
|  Set Up Date Formatting - Report Formats should already exist         |
*-----------------------------------------------------------------------*;

**** start here - modify the format using the logic above;

    proc format library=work.formats ; *** need to make this dynamic;
        value date_range(multilabel)
            
            %if &TYPE eq DAILY %then %do;
                &report_date="   Current Date: %sysfunc(putn(&report_date,mmddyy10.))"
                &previous_day="  Previous Day: %sysfunc(putn(&previous_day,mmddyy10.))"
                &same_day_last_week=" Same Day Last Week: %sysfunc(putn(&same_day_last_week,mmddyy10.))";
            %end;
        
            %if %in1(&type,DEV_DAILY DEV_MONTHLY) %then %do;
                &development_begin_date-&development_end_date="Development Period: %sysfunc(putn(&development_begin_date,mmddyy10.)) To %sysfunc(putn(&development_end_date,mmddyy10.))"; 
            %end;
        
            %if &TYPE eq MONTHLY %then %do;
                &current_month_start-&current_month_end="   Current Month: %sysfunc(putn(&current_month_start,monname.)), %sysfunc(putn(&current_month_start,year.))"
                &previous_month_start-&previous_month_end="  Previous Month: %sysfunc(putn(&previous_month_start,monname.)), %sysfunc(putn(&previous_month_start,year.))";
            %end;
    run;


    %if %sysfunc(exist(date_range)) %then %do; proc delete data=date_range; run; %end;

    proc format library=work.formats cntlout=date_range;
        select date_range;
    run;

%numobs(data=date_range,macrovar=time_periods); 

**** the following is important for type in('DAILY','MONTHLY') - the production reports ********;
%global tablecount;

proc sql noprint;
select COUNT(*)
into :tablecount
from dictionary.tables
where libname='REP' and memtype='DATA';

%put tablecount: &tablecount;

%put _user_;

%mend setup;
